"""
Utility functions
"""

import yaml
from sqlalchemy import create_engine

def make_connection(specs):
    engine = create_engine("mysql+pymysql://{user}:{pw}@{host}/{db}"
                       .format(user=specs['user'],
                               pw=specs['password'],
                               host = specs['host'],
                               db=specs['db']))
    return(engine)

def exec_file(file, add_params = None):
    """
    Executes a file 
    """
    if add_params is not None:
        exec(open(file).read(), add_params)
    else:
        exec(open(file).read())

def read_yaml(file):
    """
    A function to read a yaml file as a dictionary
    """
    with open(file, 'r') as f:
        d = yaml.load(f)
    
    return d    
        
def chunks_of_n(l, n):
    """
    Split an item into chunks of size n
    """
    for i in range(0, len(l), n):
        yield l[i:i+n]


"""
Functions for text preprocesing
"""

from nltk.corpus import stopwords
import string
import re
import inflection
from nltk.stem import WordNetLemmatizer

def lemmatize_word(string_list, engine = WordNetLemmatizer()):
    """
    Lemmatize words using word net
    """
    lemma_engine = engine
    cleaned_string = []
    
    for text in string_list:
        text = [lemma_engine.lemmatize(word) for word in text.split()]
        text = ' '.join(text)
        cleaned_string.append(text)     
        
    return cleaned_string    

def to_str(string_list):
    """
    Converts an object to string
    """
    cleaned_string = [str(x) for x in string_list]
    return cleaned_string

def rm_short_words(string_vec):
    """
    Removes words that are of length 2 or lower
    """
    cleaned_string = [re.sub(r'\b\w{1,2}\b', '', x) for x in string_vec]
    return cleaned_string
    
def to_single(string_list):
    """
    Converts a string to singular form
    """
    cleaned_string = []
    for text in string_list:
        text = [inflection.singularize(word) for word in text.split()]
        text = ' '.join(text)
        cleaned_string.append(text)
    return cleaned_string 

def to_lower(string_list):
    """
    Makes every word in a string vector lowercase
    """
    cleaned_string = [string.lower() for string in string_list]
    return cleaned_string
    
def rm_stop_words(string_list):
    """
    Removes common stop words from a string vector
    """
    cleaned_string = []
    stop_words = stopwords.words("english")   
    for text in string_list:
        text = [word for word in text.split() if word not in stop_words]
        text = ' '.join(text)
        cleaned_string.append(text)
    return cleaned_string    

def rm_punctuations(string_list):
    """
    Removes punctuations and other special characters from a string vector
    """
    cleaned_string = [re.sub(r"[^a-zA-Z0-9 ]","",s) for s in string_list]
    return cleaned_string

def rm_digits(string_list):
    """
    Removes digits from a string vector
    """
    regex = re.compile('[%s]' % re.escape(string.digits))
    cleaned_string = [regex.sub('', s) for s in string_list]
    return cleaned_string

def stem_words(string_list, stemmer):
    """
    A function to stemm the words in a given string vector
    """
    cleaned_string = []
    for text in string_list:
        text = [stemmer.stem(word) for word in text.split()]
        text = ' '.join(text)
        cleaned_string.append(text)
    return cleaned_string  

def clean_ws(string_list):
    """
    Cleans whitespaces
    """
    cleaned_string = [re.sub('\s+', ' ', s).strip() for s in string_list]
    return cleaned_string


"""
Class for data download
"""

import pandas as pd
from sqlalchemy import create_engine

class Get_sql:
    
    def get_google_tree(connection):
        """
        An sql query to download the whole google taxonomy tree 
        """
        sql_query = "SELECT distinct(full_title), id, parent_id, title as class \
                        FROM sixads.sixads_category"

        tree = pd.read_sql_query(sql_query, connection)
        def f(x):
            if x is None:
                return ""
            else:
                return '>' + x
        tree['full_title'] = [f(x) for x in tree['full_title']]
        return tree

    def get_data(connection, select_part, from_part, where_part = ''):
        "Function that construcs a query from the given parts and executes it"
        columns = ",".join(select_part)
        
        sql_query = "SELECT {columns} \
                    FROM {from_part} \
                    {where_part}".format(columns = columns, 
                                         from_part = from_part, 
                                         where_part = where_part)
                    
        return pd.read_sql_query(sql_query, connection)

    def get_labeled_data_products(connection):
        """
        An sql query to download item names with the labeled categories
        """
        sql_query = "SELECT pr.id as item_id, pr.category_id as class_id, pr.title \
                     FROM sixads.sixads_product as pr \
                     where category_id is not null"

        return pd.read_sql_query(sql_query, connection)
        
    def get_fetchproduct_data(connection, limit = None):
       """
       Downloads the data for predictions
       """
       sql_query = "SELECT id, original_title as title, shop_id \
                    FROM sixads.sixads_fetchedproduct pr"
                    
       if limit is not None :
           sql_query = sql_query + ' LIMIT ' + str(limit)
                              
       return pd.read_sql_query(sql_query, connection) 
   
    def get_excluded_cats(connection, shop_id):
        sql_query = "SELECT * \
                    FROM sixads.sixads_shop_excluded_categories \
                    where shop_id = {shop_id}".format(shop_id = shop_id)
        
        return pd.read_sql_query(sql_query, connection)
    
    def adult_info(connection, shop_id):
        sql_query = "SELECT exclude_adult \
        from sixads.sixads_shop \
        where id = {shop_id}".format(shop_id = shop_id) 
        
        return pd.read_sql_query(sql_query, connection)
    
    def curr_info(connection, shop_id):
        sql_query = "SELECT currency \
        from sixads.sixads_shop \
        where id = {shop_id}".format(shop_id = shop_id) 
        
        return pd.read_sql_query(sql_query, connection)
    
    def get_adult_cats(connection):
        sql_query = "SELECT id as category_id \
        from sixads.sixads_category \
        where base_parent_id = 3981"
        
        return pd.read_sql_query(sql_query, connection)

class Write_sql:
        
    def write_to_table(specs, table, data, if_exists = 'replace'):
        """
        Writes data to the desired table 
        """
        engine = create_engine("mysql+pymysql://{user}:{pw}@{host}/{db}"
                       .format(user=specs['user'],
                               pw=specs['password'],
                               host = specs['host'],
                               db=specs['db']))
        data.to_sql(table, con = engine, 
                            if_exists = if_exists, index = False)    


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="SixAdsDS",
    version="0.0.5",
    author="Eligijus Bujokas",
    author_email="eligijus@sixads.net",
    description="Generic functions for SixAds data science projects",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://bitbucket.org/eligijus112/sixadsml/",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires = [ 
        'inflection',
        'nltk', 
        'pymysql',
        'pandas',
        'sqlalchemy',
        'PyYAML'            
            ]        
)  
